library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity nios2_sys_test is 
port(
	-- global signals
	signal sys_clk	: in std_logic;
	signal sdram_clk: out std_logic;

	-- parallel outputs
	signal io		: out std_logic_vector(165 downto 1) ;
	signal pll1_io	: out std_logic;
	signal pll4_io	: out std_logic;

	-- user LEDs
	signal ULEDs	: out std_logic_vector(2 downto 0);

	-- sdram memory interface
	signal sd_CS_n	: out std_logic_vector(1 downto 0);
	signal CAS_n	: out std_logic;
	signal RAS_n	: out std_logic;
	signal CKE0_n	: out std_logic;
	signal CKE1_n	: out std_logic;	
	signal SD_WE_n	: out std_logic;
	signal ba_addr	: out std_logic_vector (1 downto 0);
	signal sd_dqm	: out std_logic_vector (7 downto 0);
	signal sd_addr	: out std_logic_vector (12 downto 0);
	signal sd_data	: inout std_logic_vector (63 downto 0);

	-- flash memory interface
	signal BYTE_n		: out std_logic;
	signal RST_n		: out std_logic;
	signal CS_n			: out std_logic;
	signal OE_n			: out std_logic;
	signal WE_n			: out std_logic;
	signal flash_addr	: out std_logic_vector (23 downto 0);
	signal flash_data	: inout std_logic_vector (7 downto 0);

	-- Davicom DM9000A interface
	signal eth_INT	: in std_logic;
	signal eth_CMD	: out std_logic;
	signal eth_CS_n	: out std_logic;
	signal eth_RD_n	: out std_logic;
	signal eth_WR_n	: out std_logic;
	signal eth_RST_n: out std_logic;
	signal eth_data	: inout std_logic_vector(15 downto 0);

	-- Cypress CY7C68013A interface
	signal EF_n		: in std_logic;								-- empty flag (low active)
	signal FF_n		: in std_logic;								-- full flag (low active)

	signal SLOE_n	: out std_logic;							-- slave output enable (low active)
	signal SLRD_n	: buffer std_logic;							-- slave read (low active)
	signal SLWR_n	: buffer std_logic;							-- slave write (low active)
	signal PKTEND_n	: out std_logic;							-- packet end (low active)

	signal FD		: inout std_logic_vector(15 downto 0);		-- fifo data
	signal FIFOADR	: buffer std_logic_vector(1 downto 0));		-- fifo address
end entity nios2_sys_test;

architecture structural of nios2_sys_test is

	component pll is
	port(
		inclk0	: in std_logic := '0';
		c0		: out std_logic;
		c1		: out std_logic;
		c2		: out std_logic;
		locked	: out std_logic);
	end component;

	component FIFO
		port
		(
			aclr	: in std_logic;
			clock	: in std_logic;
			data	: in std_logic_vector(15 downto 0);
			rdreq	: in std_logic;
			wrreq	: in std_logic;
			empty	: out std_logic;
			full	: out std_logic;
			q		: out std_logic_vector(15 downto 0)
		);
	end component;
	
	component nios2_sys is 
	port (
		-- global signals:
		signal clk : IN STD_LOGIC;
		signal reset_n : IN STD_LOGIC;

		-- the_dm9000a
		signal CMD_from_the_dm9000a : OUT STD_LOGIC;
		signal CS_n_from_the_dm9000a : OUT STD_LOGIC;
		signal INT_to_the_dm9000a : IN STD_LOGIC;
		signal IORD_n_from_the_dm9000a : OUT STD_LOGIC;
		signal IOWR_n_from_the_dm9000a : OUT STD_LOGIC;
		signal RST_n_from_the_dm9000a : OUT STD_LOGIC;
		signal data_to_and_from_the_dm9000a : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);

		-- the_leds
		signal out_port_from_the_leds : OUT STD_LOGIC_VECTOR (2 DOWNTO 0);

		-- the_pio_0
		signal out_port_from_the_pio_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);

		-- the_pio_1
		signal out_port_from_the_pio_1 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);

		-- the_pio_2
		signal out_port_from_the_pio_2 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);

		-- the_pio_3
		signal out_port_from_the_pio_3 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);

		-- the_pio_4
		signal out_port_from_the_pio_4 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);

		-- the_pio_5
		signal out_port_from_the_pio_5 : OUT STD_LOGIC_VECTOR (6 DOWNTO 0);

		-- the_sdram
		signal zs_addr_from_the_sdram : OUT STD_LOGIC_VECTOR (12 DOWNTO 0);
		signal zs_ba_from_the_sdram : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		signal zs_cas_n_from_the_sdram : OUT STD_LOGIC;
		signal zs_cke_from_the_sdram : OUT STD_LOGIC;
		signal zs_cs_n_from_the_sdram : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		signal zs_dq_to_and_from_the_sdram : INOUT STD_LOGIC_VECTOR (63 DOWNTO 0);
		signal zs_dqm_from_the_sdram : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		signal zs_ras_n_from_the_sdram : OUT STD_LOGIC;
		signal zs_we_n_from_the_sdram : OUT STD_LOGIC;

		-- the_tri_state_bridge_avalon_slave
		signal select_n_to_the_flash : OUT STD_LOGIC;
		signal tri_state_bridge_address : OUT STD_LOGIC_VECTOR (23 DOWNTO 0);
		signal tri_state_bridge_data : INOUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		signal tri_state_bridge_readn : OUT STD_LOGIC;
		signal write_n_to_the_flash : OUT STD_LOGIC);
	end component;

	signal clk, reset_n, cke_n, usb_clk : std_logic;
	signal output_bus	: std_logic_vector(166 downto 0);
	
	signal fifo26_full, fifo26_empty, fifo26_rd, fifo26_wr : std_logic;
	signal fifo48_full, fifo48_empty, fifo48_rd, fifo48_wr : std_logic;
	signal fifo26_out, fifo48_out	: std_logic_vector(15 downto 0);
begin
	RST_n <= '1';
	BYTE_n <= '0';
	CKE0_n <= cke_n;
	CKE1_n <= cke_n;
--	io <= output_bus(164 downto 0);
	
	io(83 downto 41) <= output_bus(42 downto 0);

	io(27 downto 21) <= output_bus(49 downto 43);
	io(33)			<= output_bus(50);					-- bug
	io(32 downto 28) <= output_bus(55 downto 51);
	io(40 downto 34) <= output_bus(62 downto 56);

	io(20)			<= output_bus(63);					-- bug
	io(19 downto 1) <= output_bus(82 downto 64);

	io(96 downto 84) <= output_bus(95 downto 83);
	pll4_io <= output_bus(96);							-- bug
	io(102 downto 97) <= output_bus(102 downto 97);

	io(123 downto 103) <= output_bus(123 downto 103);
	io(130 downto 124) <= output_bus(130 downto 124);
	pll1_io <= output_bus(131);
	io(165 downto 131) <= output_bus(166 downto 132);

	pll_inst : pll
	port map(
		inclk0	=> sys_clk,
		c0		=> clk,		
		c1		=> usb_clk,
		c2		=> sdram_clk,
		locked	=> reset_n);

	system : nios2_sys
	port map(
		-- global signals:
		clk		=> clk,
		reset_n => reset_n,

		-- the_dm9000a
		CMD_from_the_dm9000a => eth_CMD,
		CS_n_from_the_dm9000a => eth_CS_n,
		INT_to_the_dm9000a => eth_INT,
		IORD_n_from_the_dm9000a => eth_RD_n,
		IOWR_n_from_the_dm9000a => eth_WR_n,
		RST_n_from_the_dm9000a => eth_RST_n,
		data_to_and_from_the_dm9000a => eth_data,

		-- user LEDs
		out_port_from_the_leds		=> ULEDs,

		-- pio
		out_port_from_the_pio_0 	=> output_bus(31 downto 0),
		out_port_from_the_pio_1		=> output_bus(63 downto 32),
		out_port_from_the_pio_2		=> output_bus(95 downto 64),
		out_port_from_the_pio_3		=> output_bus(127 downto 96),
		out_port_from_the_pio_4		=> output_bus(159 downto 128),
		out_port_from_the_pio_5		=> output_bus(166 downto 160),

		-- the_sdram_0
		zs_addr_from_the_sdram		=> sd_addr,
		zs_ba_from_the_sdram		=> ba_addr,
		zs_cas_n_from_the_sdram		=> CAS_n,
		zs_cke_from_the_sdram		=> cke_n,
		zs_cs_n_from_the_sdram		=> sd_CS_n,
		zs_dq_to_and_from_the_sdram	=> sd_data,
		zs_dqm_from_the_sdram		=> sd_dqm,
		zs_ras_n_from_the_sdram		=> RAS_n,
		zs_we_n_from_the_sdram		=> SD_WE_n,

		-- the_tri_state_bridge_0_avalon_slave
		select_n_to_the_flash		=> CS_n,
		tri_state_bridge_address	=> flash_addr,
		tri_state_bridge_data		=> flash_data,
		tri_state_bridge_readn		=> OE_n,
		write_n_to_the_flash		=> WE_n);
		
	------------------------------------------------------------------------------------------
	SLOE_n <= SLRD_n;						-- can be asserted together
	PKTEND_n <= '1';						-- not used

	FD <= fifo26_out when fifo26_rd='1' else-- drive outputs only when needed
		  fifo48_out when fifo48_rd='1' else (others=>'Z');

	fifo26_wr <= '1' when EF_n ='1' and fifo26_full='0' and FIFOADR="00" else '0';	-- EP2 OUT endpoint (lesen)
	fifo48_wr <= '1' when EF_n ='1' and fifo48_full='0' and FIFOADR="01" else '0';	-- EP4 OUT endpoint (lesen)

	fifo26_rd <= '1' when FF_n ='1' and fifo26_empty='0' and FIFOADR="10" else '0';	-- EP6 IN endpoint (schreiben)
	fifo48_rd <= '1' when FF_n ='1' and fifo48_empty='0' and FIFOADR="11" else '0';	-- EP8 IN endpoint (schreiben)

	SLRD_n <= '0' when fifo26_wr='1' or fifo48_wr='1' else '1';
	SLWR_n <= '0' when fifo26_rd='1' or fifo48_rd='1' else '1';

	process(reset_n, usb_clk)
	begin
		if reset_n='0' then
			FIFOADR <= (others=>'0');
		else
			if usb_clk'event and usb_clk ='1' then
				if SLRD_n='1' and SLWR_n='1' then
					FIFOADR <= FIFOADR + "01";
				end if; 
			end if;
		end if;
	end process;

	fifo26: FIFO
	port map(
		aclr	=> not(reset_n),
		clock	=> usb_clk,
		data	=> FD,
		rdreq	=> fifo26_rd,
		wrreq	=> fifo26_wr,
		empty	=> fifo26_empty,
		full	=> fifo26_full,
		q		=> fifo26_out);

	fifo48: FIFO
	port map(
		aclr	=> not(reset_n),
		clock	=> usb_clk,
		data	=> FD,
		rdreq	=> fifo48_rd,
		wrreq	=> fifo48_wr,
		empty	=> fifo48_empty,
		full	=> fifo48_full,
		q		=> fifo48_out);
		
end structural;